package name.nathanmcrae.numbersstation;

import com.leakyabstractions.result.api.Result;
import com.leakyabstractions.result.core.Results;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Random;
import java.util.ResourceBundle;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.control.Alert;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.KeyCode;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Modality;
import javafx.stage.Stage;

public class MainController implements Initializable {
    private static final Logger logger = Logger.getLogger(Main.class.getName());

    private Stage settingsStage;
    private Stage selectStationStage;
    private MainSettings settings;
    private StationSettings selectedStation;

    private StringProperty selectedStationName = new SimpleStringProperty();

    @FXML
    private Label lastRetrievedLabel;

    @FXML
    private TextArea messageTextArea;

    @FXML
    private TextField stationNameField;

    @FXML
    private void handleSettingsButtonPress() {
        System.out.println("Button pressed!");
        lastRetrievedLabel.setText("Button pressed!");

        if (settingsStage == null || !settingsStage.isShowing()) {
            try {
                FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("StationSettingsView.fxml"));
                Parent root = fxmlLoader.load();

                StationSettingsController controller = fxmlLoader.getController();
                if (selectedStation == null) {
                    Alert alert = new Alert(Alert.AlertType.WARNING);
                    alert.setTitle("Invalid Station Selected");
                    alert.setHeaderText(null);
                    alert.setContentText("Invalid station selected. Please choose a station.");
                    alert.showAndWait();
                    return;
                }
                controller.setStationSettings(selectedStation);

                settingsStage = new Stage();
                settingsStage.initModality(Modality.APPLICATION_MODAL);
                settingsStage.setTitle("Numbers Station Settings");
                settingsStage.setScene(new Scene(root));
                settingsStage.show();
                settingsStage.setOnHiding(event -> {
                    updateStationSettings(selectedStation);
                });
            } catch (Exception e) {
                logger.log(Level.SEVERE, "Exception while opening settings", e);
            }
        } else {
            settingsStage.toFront();
        }
    }

    @FXML
    private void handleSelectStationButtonPress() throws Exception {
        if (selectStationStage == null || !selectStationStage.isShowing()) {
            try {
                FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("StationSelectionView.fxml"));
                Parent root = fxmlLoader.load();

                StationSelectionController controller = fxmlLoader.getController();
                controller.setSettings(settings);

                selectStationStage = new Stage();
                selectStationStage.initModality(Modality.APPLICATION_MODAL);
                selectStationStage.setUserData("test");
                selectStationStage.setTitle("Numbers Station Selection");
                selectStationStage.setScene(new Scene(root));
                selectStationStage.show();
                selectStationStage.setOnHiding(event -> {
                    String newStationName = (String)selectStationStage.getUserData();
                    StationSettings newSelectedStation = settings.getStations().stream()
                            .filter(station -> station.getName().equals(newStationName))
                            .findFirst()
                            .orElse(null);
                    updateStationSettings(newSelectedStation);
                });
            } catch (Exception e) {
                logger.log(Level.SEVERE, "Exception while selecting station", e);
            }
        } else {
            selectStationStage.toFront();
        }
    }

    @FXML
    private void handleSetAsNextMessageButtonPress() {
        Path stationDirPath = selectedStation.stationPath();

        try {
            if (!Files.exists(stationDirPath)) {
                Files.createDirectories(stationDirPath);
            }

            Path nextMessagePath = stationDirPath.resolve("next-message.txt");

            Files.write(nextMessagePath, messageTextArea.getText().getBytes());
        } catch (IOException e) {
            String message = "Failed to write next message to " + stationDirPath.toString();
            logger.log(Level.SEVERE, message, e);
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Exception");
            alert.setHeaderText(null);
            alert.setContentText(message);
            alert.showAndWait();
        }
    }

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        stationNameField.textProperty().bindBidirectional(selectedStationName);

        Result<MainSettings, Exception> result = MainSettings.load();
        if (!result.hasSuccess()) {
            // TODO: on failure, prompt user to re-initialize settings
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Settings load error");
            alert.setHeaderText(null);
            alert.setContentText("Unable to load settings file");
            alert.showAndWait();

            return;
        }

        settings = result.getSuccess().get();

        selectedStationName.set(settings.getSelectedStationName());

        if (selectedStationName.get() == null || selectedStationName.get() == "") {
            selectedStation = settings.getStations().stream().findFirst().orElse(null);
            selectedStationName.set(selectedStation.getName());
        }

        selectedStation = settings.getStations().stream()
            .filter(station -> station.getName().equals(selectedStationName.get()))
            .findFirst()
            .orElse(null);

        if (selectedStation == null) {
            // TODO: create a new station and save it?
        }

        messageTextArea.addEventFilter(KeyEvent.ANY, event -> {
            int cursorPosition = messageTextArea.getCaretPosition();

            String character = event.getCharacter();
            KeyCode code = event.getCode();

            String str = event.getText();

            // Consume the event to block the default behavior
            if (!(code == KeyCode.LEFT ||
                  code == KeyCode.RIGHT ||
                  code == KeyCode.UP ||
                  code == KeyCode.DOWN ||
                  code == KeyCode.HOME ||
                  code == KeyCode.END ||
                  code == KeyCode.PAGE_UP ||
                  code == KeyCode.PAGE_DOWN)) {
                event.consume();
            }

            if (event.getEventType() == KeyEvent.KEY_PRESSED &&
                (code == KeyCode.DIGIT0 ||
                 code == KeyCode.DIGIT1 ||
                 code == KeyCode.DIGIT2 ||
                 code == KeyCode.DIGIT3 ||
                 code == KeyCode.DIGIT4 ||
                 code == KeyCode.DIGIT5 ||
                 code == KeyCode.DIGIT6 ||
                 code == KeyCode.DIGIT7 ||
                 code == KeyCode.DIGIT8 ||
                 code == KeyCode.DIGIT9)) {

                if (cursorPosition < messageTextArea.getLength()) {
                    char currentChar = messageTextArea.getText().charAt(cursorPosition);
                    if (Character.isWhitespace(currentChar)) {
                        messageTextArea.replaceText(cursorPosition + 1, cursorPosition + 2, str);
                    } else {
                        messageTextArea.replaceText(cursorPosition, cursorPosition + 1, str);
                    }
                }
            }
        });
    }

    private void updateStationSettings(StationSettings newStationSettings) {
        if (newStationSettings == null) {
            return;
        }

        logger.info("Updating station settings: " + newStationSettings.getName());

        selectedStation = newStationSettings;
        selectedStationName.set(newStationSettings.getName());
        settings.setSelectedStationName(selectedStationName.get());

        settings.save();

        // TODO: Load message from file
        // If the message we're overwriting is different from its file, then prompt to confirm

        Random random = new Random();
        StringBuilder messageBuilder = new StringBuilder();

        for (int i = 0; i < selectedStation.getMessageLength(); i++) {
            if (i > 0 && i % selectedStation.getDigitsPerGroup() == 0) {
                messageBuilder.append(" ");
            }
            messageBuilder.append(random.nextInt(10));
        }

        messageTextArea.setText(messageBuilder.toString());
    }
}
